#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

# import standard library

# import interfaces
from onf_website.component.hearing.interfaces import IHearingManagerTarget, IHearingManagerSettings, \
    HEARING_SETTINGS_KEYS, HEARING_SETTINGS
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_form.interfaces.form import IWidgetsSuffixViewletsManager, IFormHelp
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn

# import packages
from onf_website.component.hearing.manager import HearingSetting
from onf_website.skin import onf_website
from pyams_form.help import FormHelp
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.table import BaseTable, I18nColumn, JsActionColumn, TableViewlet
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogDisplayForm
from pyramid.exceptions import NotFound
from pyramid.view import view_config
from z3c.table.column import GetAttrColumn

from onf_website import _


@viewlet_config(name='hearing-settings.menu', context=IHearingManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_TOOL_PERMISSION, weight=30)
class HearingManagerSettingsMenu(MenuItem):
    """Hearing manager settings menu"""

    label = _("Hearing settings...")
    icon_class = 'fa-volume-up'
    url = 'hearing-settings.html'
    modal_target = True


@pagelet_config(name='hearing-settings.html', context=IHearingManagerTarget, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class HearingManagerSettingsView(AdminDialogDisplayForm):
    """Hearing manager settings view"""

    legend = _("Set access to hearing settings for this tool")


class HearingManagerSettingsTable(BaseTable):
    """Hearing manager settings table"""

    id = 'hearing_manager_table'
    title = _("Hearing settings")
    hide_toolbar = True
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight datatable'}
    sortOn = None

    @property
    def data_attributes(self):
        attributes = super(HearingManagerSettingsTable, self).data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-plugins': 'onf_website',
            'data-ams-plugin-onf_website-src': get_resource_path(onf_website),
            'data-ams-location': absolute_url(self.context, self.request),
            'data-ams-datatable-global-filter': 'false',
            'data-ams-datatable-info': 'false',
            'data-ams-datatable-sort': 'false',
            'data-ams-datatable-sdom': "t"
        })
        attributes.setdefault('tr', {}).update({
            'id': lambda x, col: '{0}::{1}'.format(self.id, x.name),
            'data-ams-element-name': lambda x, col: x.name
        })
        return attributes


@adapter_config(name='name', context=(IHearingManagerTarget, IAdminLayer, HearingManagerSettingsTable),
                provides=IColumn)
class HearingManagerSettingsNameColumn(I18nColumn, GetAttrColumn):
    """Hearing manager settings name column"""

    _header = _("Setting type")
    attrName = 'name'
    weight = 10

    def getValue(self, item):
        value = super(HearingManagerSettingsNameColumn, self).getValue(item)
        translate = self.request.localizer.translate
        return translate(HEARING_SETTINGS[value])


@adapter_config(name='visibility', context=(IHearingManagerTarget, IAdminLayer, HearingManagerSettingsTable),
                provides=IColumn)
class HearingManagerSettingsVisibleColumn(JsActionColumn):
    """Hearing manager settings visibility column"""

    cssClasses = {'th': 'action',
                  'td': 'action switcher'}

    icon_class = 'fa fa-fw fa-eye'
    icon_hint = _("Set access or not to this setting in contents")

    url = 'ONF_website.hearing.switchVisibility'
    weight = 80

    def get_icon_class(self, item):
        if item.visible:
            return self.icon_class
        else:
            return 'fa fa-fw fa-eye-slash text-danger'


@adapter_config(name='folding', context=(IHearingManagerTarget, IAdminLayer, HearingManagerSettingsTable),
                provides=IColumn)
class HearingManagerSettingsFoldingColumn(JsActionColumn):
    """Hearing manager settings folding column"""

    cssClasses = {'th': 'action',
                  'td': 'action switcher'}

    icon_class = 'fa fa-fw fa-folder-open-o'
    icon_hint = _("Pre-unfold or not this setting in contents")

    url = 'ONF_website.hearing.switchUnfolded'
    weight = 90

    def get_icon_class(self, item):
        if item.unfolded:
            return self.icon_class
        else:
            return 'fa fa-fw fa-folder-o'


@adapter_config(context=(IHearingManagerTarget, IAdminLayer, HearingManagerSettingsTable), provides=IValues)
class HearingManagerSettingsValuesAdapter(ContextRequestViewAdapter):
    """Hearing manager settings table values adapter"""

    @property
    def values(self):
        settings = IHearingManagerSettings(self.context)
        for key in HEARING_SETTINGS_KEYS:
            value = settings.get(key)
            if value is None:
                value = settings[key] = HearingSetting(key)
            yield value


@viewlet_config(name='hearing-settings-table', view=HearingManagerSettingsView, layer=IAdminLayer,
                manager=IWidgetsSuffixViewletsManager)
class HearingManagerSettingsTableViewlet(TableViewlet):
    """Hearing manager settings table viewlet"""

    table_class = HearingManagerSettingsTable


@view_config(name='switch-hearing-setting-visibility.json', context=IHearingManagerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
def switch_hearing_setting_visibility(request):
    """Set hearing setting visibility"""
    settings = IHearingManagerSettings(request.context)
    object_name = request.params.get('object_name')
    setting = settings.get(object_name)
    if setting is None:
        raise NotFound()
    setting.visible = not setting.visible
    settings[object_name] = setting  # for persistance...
    return {'visible': setting.visible}


@view_config(name='switch-hearing-setting-folding.json', context=IHearingManagerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
def switch_hearing_setting_folding(request):
    """Set hearing setting folding"""
    settings = IHearingManagerSettings(request.context)
    object_name = request.params.get('object_name')
    setting = settings.get(object_name)
    if setting is None:
        raise NotFound()
    setting.unfolded = not setting.unfolded
    settings[object_name] = setting  # for persistance...
    return {'unfolded': setting.unfolded}


@adapter_config(context=(IHearingManagerTarget, IAdminLayer, HearingManagerSettingsView), provides=IFormHelp)
class HearingManagerSettingsViewHelp(FormHelp):
    """Hearing settings view form help"""

    message = _("Hearing settings are not required for each type of content.\n\n"
                "This form allows you to define which settings are available for this content type, and if so "
                "which ones are unfolded by default.")
    message_format = 'rest'
