#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.location.interfaces import ILocationInfo, ILocationTarget, LOCATION_INFO_KEY, \
    ILocationManagerTarget, ILocationLocationInfo, ILocationInseeInfo
from onf_website.reference.location.interfaces.region import IRegion
from pyams_content.features.checker.interfaces import IContentChecker, ERROR_VALUE
from pyams_content_es.interfaces import IDocumentIndexInfo
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.annotation.interfaces import IAnnotations

# import packages
from onf_website.reference.forest.model.foret import PARENT_SESSION as RDF_PARENT_SESSION, Foret
from persistent import Persistent
from pyams_alchemy.engine import get_user_session
from pyams_content.features.checker import BaseContentChecker
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.inherit import BaseInheritInfo, InheritedFieldProperty
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import volatile_property
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent, IObjectMovedEvent, IObjectModifiedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty

from onf_website import _


@implementer(ILocationInfo)
class LocationInfo(BaseInheritInfo, Persistent, Contained):
    """Location info persistent class"""

    adapted_interface = ILocationInfo
    target_interface = ILocationTarget

    _gps_location = FieldProperty(ILocationInfo['gps_location'])
    gps_location = InheritedFieldProperty(ILocationInfo['gps_location'])
    _forests = FieldProperty(ILocationInfo['forests'])
    forests = InheritedFieldProperty(ILocationInfo['forests'])
    _all_departments_forests = FieldProperty(ILocationInfo['all_departments_forests'])
    all_departments_forests = InheritedFieldProperty(ILocationInfo['all_departments_forests'])
    _cities = FieldProperty(ILocationInfo['cities'])
    cities = InheritedFieldProperty(ILocationInfo['cities'])
    _address = FieldProperty(ILocationInfo['address'])
    address = InheritedFieldProperty(ILocationInfo['address'])
    _situation = FieldProperty(ILocationInfo['situation'])
    situation = InheritedFieldProperty(ILocationInfo['situation'])
    _natural_region = FieldProperty(ILocationInfo['natural_region'])
    natural_region = InheritedFieldProperty(ILocationInfo['natural_region'])
    _forests_label = FieldProperty(ILocationInfo['forests_label'])
    forests_label = InheritedFieldProperty(ILocationInfo['forests_label'])
    _departments = FieldProperty(ILocationInfo['departments'])
    departments = InheritedFieldProperty(ILocationInfo['departments'])
    _countries = FieldProperty(ILocationInfo['countries'])
    countries = InheritedFieldProperty(ILocationInfo['countries'])
    _structures = FieldProperty(ILocationInfo['structures'])
    structures = InheritedFieldProperty(ILocationInfo['structures'])

    @volatile_property
    def forests_index(self):
        forests = set(self.forests or [])
        if self.departments and self.all_departments_forests:
            session = get_user_session(RDF_PARENT_SESSION)
            results = set(*zip(*session.query(Foret.id_nat_frt)
                                       .filter(Foret.departement.in_(self.departments_index))))
            if results:
                forests |= results
        return list(forests)

    def get_departments(self):
        for value in self.departments or ():
            if IRegion.providedBy(value):
                for dept in value.departments or ():
                    yield dept
            else:
                yield value

    @volatile_property
    def departments_index(self):
        return [dept.insee_code for dept in self.get_departments()]

    @volatile_property
    def countries_index(self):
        return [value.insee_code for value in self.countries or ()]


@adapter_config(context=ILocationTarget, provides=ILocationInfo)
def LocationInfoFactory(target):
    """Location info factory"""
    annotations = IAnnotations(target)
    location = annotations.get(LOCATION_INFO_KEY)
    if location is None:
        location = annotations[LOCATION_INFO_KEY] = LocationInfo()
        get_current_registry().notify(ObjectCreatedEvent(location))
        locate(location, target, '++location++')
    return location


@adapter_config(context=ILocationInfo, provides=IFormContextPermissionChecker)
class LocationInfoPermissionChecker(ContextAdapter):
    """Location info checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, ILocationTarget)
        return IFormContextPermissionChecker(content).edit_permission


@adapter_config(name='location', context=ILocationTarget, provides=IDocumentIndexInfo)
def LocationTargetIndexInfo(content):
    """Location index info"""
    location = ILocationInfo(content)
    values = {
        'forests': location.forests_index,
        'cities': location.cities,
        'departments': location.departments_index,
        'countries': location.countries_index,
        'structures': location.structures
    }
    if location.gps_location:
        values['coords'] = location.gps_location.wgs_coordinates
    return {'location': values}


@adapter_config(name='location', context=ILocationTarget, provides=IContentChecker)
class LocationContentChecker(BaseContentChecker):
    """Location content checker"""

    label = _("Location")
    weight = 90

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        location = ILocationInfo(self.context)
        if not (location.countries or location.departments or location.structures or
                location.address or location.cities or location.forests or location.gps_location):
            output.append(translate(ERROR_VALUE).format(field=translate(_("Location fields")),
                                                        message=translate(_("no defined location"))))
        return output


@subscriber(IObjectModifiedEvent, context_selector=ILocationTarget)
def handle_modified_location(event):
    """Delete volatile attributes when location is modified"""
    location = ILocationInfo(event.object)
    for attributes in event.descriptions:
        if attributes.interface is ILocationLocationInfo:
            del location.forests_index
        elif attributes.interface is ILocationInseeInfo:
            del location.departments_index
            del location.countries_index


@subscriber(IObjectMovedEvent, context_selector=ILocationTarget)
def handle_moved_location(event):
    """Remove volatile 'parent' property when location target is moved"""
    location = ILocationInfo(event.object)
    del location.parent
